@@ -72,7 +72,7 @@ class TicketLengthError(CryptoError):
         super().__init__(length)
 
     def __str__(self):
-        return f'0x350 expected, {self.args[0]:#x} given'
+        return f'0x350 expected, {hex(self.args[0])} given'
 
 
 # wonder if I'm doing this right...
@@ -142,6 +142,12 @@ class Keyslot(IntEnum):
     DecryptedTitlekey = 0x40
     """CIA and CDN contents."""
 
+    ZeroKey = 0x41
+    """All zero key for NCCH titles using fixed crypto."""
+
+    FixedSystemKey = 0x42
+    """Special key for NCCH system titles using fixed crypto."""
+
 
 common_key_y = (
     # eShop
@@ -249,9 +255,10 @@ class CryptoEngine:
 
     def __init__(self, boot9: str = None, dev: bool = False, setup_b9_keys: bool = True):
         self.key_x: Dict[int, int] = {}
-        self.key_y: Dict[int, int] = {0x03: 0xE1A00005202DDD1DBD4DC4D30AB9DC76,
-                                      0x05: 0x4D804F4E9990194613A204AC584460BE}
-        self.key_normal: Dict[int, bytes] = {}
+        self.key_y: Dict[int, int] = {Keyslot.TWLNAND: 0xE1A00005202DDD1DBD4DC4D30AB9DC76,
+                                      Keyslot.CTRNANDNew: 0x4D804F4E9990194613A204AC584460BE}
+        self.key_normal: Dict[int, bytes] = {Keyslot.ZeroKey: b'\0' * 16,
+                                             Keyslot.FixedSystemKey: bytes.fromhex('527CE630A9CA305F3696F3CDE954194B')}
 
         self.dev = dev
 
@@ -619,7 +626,7 @@ class CryptoEngine:
         otp_keysect_hash: bytes = sha256(otp_enc[0:0x90]).digest()
 
         self.set_keyslot('x', 0x11, otp_keysect_hash[0:0x10])
-        self.set_keyslot('y', 0x11, otp_keysect_hash[0:0x10])
+        self.set_keyslot('y', 0x11, otp_keysect_hash[0x10:0x20])
 
         # most otp code from https://github.com/Stary2001/3ds_tools/blob/master/three_ds/aesengine.py
 
@@ -698,7 +705,7 @@ class CryptoEngine:
         elif len(data) in {0x120, 0x140}:
             key = data[0x110:0x120]
         else:
-            raise BadMovableSedError(f'invalid length ({len(data):#x}')
+            raise BadMovableSedError(f'invalid length ({hex(len(data))}')
 
         self.set_keyslot('y', Keyslot.SD, key)
         self.set_keyslot('y', Keyslot.CMACSDNAND, key)
@@ -761,7 +768,7 @@ class CTRFileIO(_CryptoFileBase):
         self._lock = Lock()
 
     def __repr__(self):
-        return f'{type(self).__name__}(file={self._reader!r}, keyslot={self._keyslot:#04x}, counter={self._counter!r})'
+        return f'{type(self).__name__}(file={self._reader!r}, keyslot={self._keyslot}, counter={self._counter!r})'
 
     @_raise_if_file_closed
     def read(self, size: int = -1) -> bytes:
@@ -804,7 +811,7 @@ class CBCFileIO(_CryptoFileBase):
         self._lock = Lock()
 
     def __repr__(self):
-        return f'{type(self).__name__}(file={self._reader!r}, keyslot={self._keyslot:#04x}, iv={self._iv!r})'
+        return f'{type(self).__name__}(file={self._reader!r}, keyslot={self._keyslot}, iv={self._iv!r})'
 
     @_raise_if_file_closed
     def read(self, size: int = -1):
